﻿using Hotline.FlowEngine.Definitions;
using Hotline.FlowEngine.WorkflowModules;
using Hotline.FlowEngine.Workflows;
using Hotline.Orders;
using Hotline.Share.Dtos;
using Hotline.Share.Dtos.File;
using Hotline.Share.Dtos.FlowEngine;
using Hotline.Share.Dtos.FlowEngine.Definition;
using Hotline.Share.Dtos.Order;
using Hotline.Share.Dtos.Settings;
using Hotline.Share.Enums.FlowEngine;

namespace Hotline.Application.FlowEngine
{
    public interface IWorkflowApplication
    {


        /// <summary>
        /// 开始流程
        /// </summary>
        Task<string> StartWorkflowAsync(StartWorkflowDto dto, string externalId, DateTime? expiredTime = null,
            CancellationToken cancellationToken = default);

        /// <summary>
        /// 流转至下一节点（节点办理）
        /// </summary>
        Task<Workflow> NextAsync(NextWorkflowDto dto, DateTime? expiredTime = null, CancellationToken cancellationToken = default);

        /// <summary>
        /// 退回（返回前一节点）
        /// </summary>
        Task PreviousAsync(PreviousWorkflowDto dto, CancellationToken cancellationToken);

        /// <summary>
        /// 工单退回（返回前一节点）
        /// </summary>
        Task PreviousAsync(PreviousWorkflowDto dto, string userId, CancellationToken cancellationToken);

        /// <summary>
        /// 撤回至任意节点
        /// </summary>
        Task RecallAsync(RecallDto dto, DateTime? expiredTime, CancellationToken cancellationToken);

        ///// <summary>
        ///// 跳转至任意节点
        ///// </summary>
        //Task JumpAsync(RecallDto dto, CancellationToken cancellationToken);

        /// <summary>
        /// 跳转至结束节点(无视流程模板配置直接跳至结束节点)
        /// </summary>
        Task JumpToEndAsync(string workflowId, string opinion, List<FileDto> file,
            EReviewResult? reviewResult = EReviewResult.Unknown,
            CancellationToken cancellationToken = default);

        ///// <summary>
        ///// 重办
        ///// </summary>
        //Task RedoAsync(RecallDto dto, CancellationToken cancellationToken);

        /// <summary>
        /// 查询流程开始节点的下一节点配置
        /// </summary>
        Task<DefinedStepDto> GetStartOptionsAsync(string moduleCode, CancellationToken cancellationToken);

        /// <summary>
        /// 查询节点配置可选参数
        /// </summary>
        Task<IReadOnlyList<Kv>> GetNextStepOptionsAsync(StepDefineBasic stepDefine, CancellationToken cancellationToken);

        ////////

        /// <summary>
        /// 查询开始流程的下一步待选节点
        /// </summary>
        Task<NextStepsDto<NextStepOption>> GetStartStepsAsync(string moduleCode, CancellationToken cancellationToken);

        /// <summary>
        /// 查询办理流程的下一步待选节点
        /// </summary>
        Task<NextStepsDto<NextStepOption>> GetNextStepsAsync(string workflowId, CancellationToken cancellationToken);

        /// <summary>
        /// 查询撤回可选节点
        /// </summary>
        Task<NextStepsDto<RecallStepOption>> GetRecallStepsAsync(string workflowId, CancellationToken cancellationToken);

        /// <summary>
        /// 查询跳转可选节点
        /// </summary>
        Task<NextStepsDto> GetJumpStepsAsync(string workflowId, CancellationToken cancellationToken);

        /// <summary>
        /// 查询重办可选节点
        /// </summary>
        Task<NextStepsDto> GetRedoStepsAsync(string workflowId, CancellationToken cancellationToken);

        /// <summary>
        /// 否决
        /// </summary>
        Task RejectAsync(RejectDto dto, CancellationToken cancellationToken);

        /// <summary>
        /// 获取流程模板信息
        /// </summary>
        /// <param name="code"></param>
        /// <param name="cancellationToken"></param>
        /// <returns></returns>
        Task<WorkflowModule> GetWorkflowModuleAsync(string code, CancellationToken cancellationToken);
    }
}